<?php
//
// Copyright (c) 2011 Nagios Enterprises, LLC.  All rights reserved.
//
// $Id: mibs.php 451 2011-01-13 18:04:47Z egalstad $

require_once(dirname(__FILE__) . '/../includes/common.inc.php');

// initialization stuff
pre_init();

// start session
init_session();

// grab GET or POST variables 
grab_request_vars();

// check prereqs
check_prereqs();

// check authentication
check_authentication(false);

// only admins can access this page
if (is_admin() == false) {
    echo $lstr['NotAuthorizedErrorText'];
    exit();
}

// route request
route_request();


function route_request()
{
    global $request;

    //if(in_demo_mode()==true)
    //header("Location: main.php");

    if (isset($request["download"]))
        do_download();
    else if (isset($request["upload"])) {
        $processMib = false;
        if (isset($request["processMibCheck"]) && $request["processMibCheck"] == 'YES')
            $processMib = true;
        do_upload($processMib);
    } else if (isset($request["delete"]))
        do_delete();
    else
        show_mibs();

    exit;
}


/**
 * @param bool   $error
 * @param string $msg
 */
function show_mibs($error = false, $msg = "")
{
    global $lstr;


    $mibs = get_mibs();


    do_page_start(array("page_title" => $lstr['MIBsPageTitle']), true);

    ?>


    <h1><?php echo $lstr['MIBsPageHeader']; ?></h1>


    <?php
    display_message($error, false, $msg);
    ?>

    <?php echo $lstr['MIBsPageNotes']; ?>


    <p>
    <div class="bluebutton" style="width: 150px; float: right;">
        <a href="http://www.mibdepot.com/" target="_blank"><?php echo gettext("Get MIBs"); ?></a>
    </div>
    <?php echo gettext("You can find hundreds of additional MIBs at the following sites"); ?>:
    </p>
    <ul>
        <li><a href="http://www.mibdepot.com/" target="_blank">http://www.mibdepot.com</a></li>
        <li><a href="http://www.oidview.com/mibs/detail.html" target="_blank">http://www.oidview.com/mibs/</a></li>
    </ul>

    <br clear="all">
    <br clear="all">

    <?php
    //print_r($plugins);
    ?>

    <form enctype="multipart/form-data" action="" method="post">
        <input type="hidden" name="upload" value="1">
        <?php echo get_nagios_session_protector(); ?>
        <input type="hidden" name="MAX_FILE_SIZE" value="50000000"/>
        <label><?php echo $lstr["UploadNewMIBBoxText"]; ?>:</label><br>
        <input name="uploadedfile" class="textfield" type="file"/>
        <label><?php echo gettext("Process trap"); ?>:</label><input type="checkbox" name="processMibCheck"
                                                                     value="YES"/>
        <input type="submit" class="submitbutton" value="<?php echo $lstr['UploadMIBButton']; ?>"/>
    </form>

    <br>

    <table class="standardtable">
        <thead>
        <tr>
            <th><?php echo $lstr['MIBTableHeader']; ?></th>
            <th><?php echo $lstr['FileTableHeader']; ?></th>
            <th><?php echo $lstr['FileOwnerTableHeader']; ?></th>
            <th><?php echo $lstr['FileGroupTableHeader']; ?></th>
            <th><?php echo $lstr['FilePermsTableHeader']; ?></th>
            <th><?php echo $lstr['DateTableHeader']; ?></th>
            <th><?php echo $lstr['ActionsTableHeader']; ?></th>
        </tr>
        </thead>
        <tbody>

        <?php
        $x = 0;
        foreach ($mibs as $mib) {

            $x++;

            $rowclass = "";

            if (($x % 2) != 0)
                $rowclass .= " odd";
            else
                $rowclass .= " even";

            echo "<tr class=" . $rowclass . ">";
            echo "<td>" . $mib["mibname"] . "</td>";
            echo "<td>" . $mib["file"] . "</td>";
            echo "<td>" . $mib["owner"] . "</td>";
            echo "<td>" . $mib["group"] . "</td>";
            echo "<td>" . $mib["permstring"] . "</td>";
            echo "<td>" . $mib["date"] . "</td>";
            echo "<td>";
            echo "<a href='?download=" . $mib["file"] . "'><img src='" . theme_image("download.png") . "' alt='" . $lstr['DownloadAlt'] . "' title='" . $lstr['DownloadAlt'] . "'></a> ";
            echo "<a href='?delete=" . $mib["file"] . "&nsp=" . get_nagios_session_protector_id() . "'><img src='" . theme_image("delete.png") . "' alt='" . $lstr['DeleteAlt'] . "' title='" . $lstr['DeleteAlt'] . "'></a>";
            echo "</td>";
            echo "</tr>\n";
        }
        ?>

        </tbody>
    </table>

    <?php

    do_page_end(true);
    exit();
}


function do_download()
{

    $file = grab_request_var("download", "");


    // clean the filename
    $file = str_replace("..", "", $file);
    $file = str_replace("/", "", $file);
    $file = str_replace("\\", "", $file);

    $dir = get_mib_dir();
    $thefile = $dir . "/" . $file;

    $mime_type = "";
    header('Content-type: ' . $mime_type);
    header("Content-length: " . filesize($thefile));
    header('Content-Disposition: attachment; filename="' . basename($thefile) . '"');
    readfile($thefile);
    exit();
}

/**
 * @param $src
 * @param $dest
 * @param $message
 *
 * @return int
 */
function convert_mib($src, $dest, &$message)
{
    $rc = 0;
    $convertmib_location = '';
    $matches = array();
    $convert_results = array();
    $success_string = array();

    // locate snmpttconvertmib
    $locate_convertmib = 'which snmpttconvertmib';
    exec($locate_convertmib, $output);
    $convertmib_location = $output;

    // construct and execute command
    $convert_cmd = $convertmib_location . ' --in=' . escapeshellarg($src) . ' --out=' . escapeshellarg($dest);
    exec($convert_cmd, $convert_results, $rc);
    if ($rc != 0) {
        $message = gettext("Failed to convert uploaded file to snmptt mib.") . ". (rc: " . $rc . ")";
    } else {
        foreach ($convert_results as $line) {
            if (preg_match('/Failed translations:\s*(\d+)/', $line, $matches) === 1) {
                if ($matches[1] > 0) {
                    $rc = 1;
                    $message = gettext("Failed to convert uploaded file to snmptt mib.");
                    break;
                }
            } else if (strpos($line, "translations:") !== false) {
                $success_string[] = $line;
            } else if (strpos($line, "MIB file did not contain any TRAP-TYPE or NOTIFICATION-TYPE") !== false) {
                $rc = 2;
                $success_string[] = "MIB is added, but is does not contain any TRAP-TYPE definitions, so no traps were added, even though user input said there should be traps specified.";
                break;
            }
        }

        if ($rc === 0 || $rc === 2) {
            $message = implode("<br />", $success_string);
        }
    }
    return $rc;
}


/**
 * @return string
 */
function get_snmptt_ini_path()
{
    return '/etc/snmp/snmptt.ini';
}


/**
 * @param $mib
 * @param $message
 *
 * @return int
 */
function add_mib_to_conf($mib, &$message)
{
    //add processed mib to snmptt.ini
    $sed_status = 0;
    $sed_results = array();
    $snmptt_ini_path = get_snmptt_ini_path();

    $grep_cmd = "grep $mib $snmptt_ini_path";
    exec($grep_cmd, $grep_arry, $rc);
    if ($rc == 0) {
        return 0; //already in file
    }

    $add_to_snmpttini_cmd = "/bin/sed -i '/^snmptt_conf_files/a" . $mib . "' " . $snmptt_ini_path;
    exec($add_to_snmpttini_cmd, $sed_results, $sed_status);
    if ($sed_status != 0) {
        $message = gettext("Failed to converted mib path to snmptt.ini") . ". (rc: " . $sed_status . ")";
    }

    return $sed_status;
}

/**
 * @param $mib
 * @param $message
 *
 * @return mixed
 */
function run_addmib($mib, &$message)
{
    //run addmib command if command exists
    $addmib = '/usr/local/bin/addmib';
    if (is_executable($addmib)) {
        $cmd = exec($addmib . " " . $mib, $addmib_result, $addmib_returncode);
        if ($addmib_returncode != 0)
            $message = gettext("Failed to execute addmib command for ") . $mib . ". (rc: " . $addmib_returncode . ")";
        return $addmib_returncode;
    }
}


/**
 * @param $message
 *
 * @return int
 */
function restart_snmptt(&$message)
{
    $cmd = 'sudo /etc/init.d/snmptt restart'; //must add to sudoers.
    $rc = 0;
    exec($cmd, $results, $rc);
    if ($rc != 0) {
        $message = gettext("Failed to restart snmptt service");
    }

    return $rc;
}


/**
 * @param $processMib
 */
function do_upload($processMib)
{
    global $lstr;

    // demo mode
    if (in_demo_mode() == true)
        show_mibs(true, $lstr['DemoModeChangeError']);
    if ($processMib) {
        if (!file_exists(get_snmptt_ini_path())) {
            show_mibs(true, gettext('MIB could not be installed - snmptt is not installed.'));
            exit();
        } elseif (!is_writable(get_snmptt_ini_path())) {
            show_mibs(true, 'snmptt.ini ' . gettext('is not writable by Nagios.') . '<br>' . gettext('Run the following from the command line') . ':<br><br>chown root.nagios /etc/snmp/snmptt.ini /etc/snmp<br>
	chmod g+w /etc/snmp/snmptt.ini /etc/snmp');
            exit();

        }
    }

    // check session
    check_nagios_session_protector();

    //print_r($request);

    $target_path = get_mib_dir();
    $target_path .= "/";
    $target_path .= basename($_FILES['uploadedfile']['name']);

    //echo "TEMP NAME: ".$_FILES['uploadedfile']['tmp_name']."<BR>\n";

    if (!move_uploaded_file($_FILES['uploadedfile']['tmp_name'], $target_path)) {
        // error
        show_mibs(true, $lstr['MIBUploadFailedText']);
    } else if ($processMib) {
        chmod($target_path, 0664);
        // success!

        // process mib
        $convert_path = get_mib_dir();
        $convert_path .= "/processed_mibs/";

        // make directory if doesn't exist
        if (!is_dir($convert_path))
            mkdir($convert_path);

        $target_path_parts = pathinfo($target_path);
        $convert_path .= $target_path_parts['filename'] . '.txt';
        $convert_message = "";

        //run addmib if present
        $rc_add = run_addmib($target_path, $convert_message);

        //Process mib with convertmib if addmib isn't executable
        if ($rc_add !== 0) {
            $rc = convert_mib($target_path, $convert_path, $convert_message);
            if ($rc !== 0) {
                if ($rc === 1) {
                    unlink($convert_path);
                }
                show_mibs(true, $convert_message);
            } else if (add_mib_to_conf($convert_path, $message) != 0) {
                unlink($convert_path);
                show_mibs(true, $message);
            } else {
                $result_message = $lstr['MIBProcessedText'];
                $result_message .= $convert_message;
                if (restart_snmptt($restart_message) != 0)
                    $result_message .= "<br>" . $restart_message;

                show_mibs(false, $result_message);
            }
        }
    } else {
        // successfully uploaded file
        chmod($target_path, 0664);
        show_mibs(false, $lstr['MIBUploadedText']);
    }

    exit();
}

function do_delete()
{
    global $lstr;


    // demo mode
    if (in_demo_mode() == true)
        show_mibs(true, $lstr['DemoModeChangeError']);

    // check session
    check_nagios_session_protector();

    $file = grab_request_var("delete", "");

    // clean the filename
    $file = str_replace("..", "", $file);
    $file = str_replace("/", "", $file);
    $file = str_replace("\\", "", $file);

    $dir = get_mib_dir();
    $thefile = $dir . "/" . $file;

    if (unlink($thefile) === TRUE) {
        // success!
        show_mibs(false, $lstr['MIBDeletedText']);
    } else {
        // error
        show_mibs(true, $lstr['MIBDeleteFailedText']);
    }
}

?>
