<?php
// perfdata.inc.php
// - Template file for highcharts perfdata graphs     

// Pass all args into template and return JSON code 
/**
 * @param $args Includes height, width, hs_url (host/service details page url)
 * @return string Javascript Highcharts Object
 */
function fetch_timeline_perfdata($args)
{   
    // Allow for height & width to be passed for resizing
    $height = grab_array_var($args, 'height', 500);
    $width = grab_array_var($args, 'width');
    $hs_url = grab_array_var($args, "hs_url");
    $no_legend = grab_array_var($args, "no_legend");

    // Check width
    if (!empty($width)) {
        $width = "width: {$width},";
    }

    // Defaults
    if (empty($no_hover)) { $no_hover = ''; }
    if (empty($no_hover_fill)) { $no_hover_fill = ''; }

    // Create readable graph
    $args['title'] = str_replace("_", " ", $args['title']); // Replaces underscores with spaces
    $units = explode(" ", $args['UOM']);
    foreach ($units as &$u) { if ($u == "%%") { $u = "%"; } } 

    if (count($units) == 3) {
        if ($units[0] == $units[1] && $units[2] == "") {
            $args['UOM'] = $units[0]; // Fix double units of measurement
        }
    }

    $uom_array = array();
    for ($i = 0; $i < count($args['names']); $i++) {
        $uom_array[] = "'".$args['names'][$i]."': '".$units[$i]."'";
    }
    $uom_array_str = implode(',', $uom_array);

    // Create the AVG and MAX for each set we are going to display...
    for ($i = 0; $i < count($args['datastrings']); $i++) {
        $data = $args['datastrings'][$i];
        $max = 0;
        $avg = 0;
        $avg_total = 0;
        $avg_num = count($data);
        for ($j = 0; $j < count($data); $j++) {
            if ($data[$j] > $max) { $max = $data[$j]; }
            $avg_total += $data[$j];
        }
        $avg = $avg_total / $avg_num;
        $perfdata_extra[$i]['avg'] = $avg;
        $perfdata_extra[$i]['max'] = $max;
        $perfdata_extra[$i]['last'] = $data[$j-1];
    }

    // Check if no legend
    $legend = 'true';
    if ($no_legend == 1) {
        $legend = 'false';
    }

    // Special export settings for local exporting
    $filename = str_replace(array("  ", " ", ":", "__", "_-_"),
                            array(" ", "_", "-", "_", "-"),
                            strtolower($args['title']));
    $filename = trim($filename, "_");
    $exporting = "";
    if (get_option('highcharts_local_export_server', 1)) {
        $exporting = "exporting: {
            url: '".get_base_url()."/includes/components/highcharts/exporting-server/index.php',
            sourceHeight: $('#{$args['container']}').height(),
            sourceWidth: $('#{$args['container']}').width(),
            filename: '{$filename}',
            chartOptions: { chart: { spacing: [25, 25, 25, 20], marginRight: 30 } }
         },";
    }

    // Begin heredoc string syntax 
    $graph = <<<GRAPH
        
        var COUNT = {$args['count']}; //total rrd entries fetched 
        var UOM = '{$args['UOM']}';
        var START = {$args['start']};   //Date.UTC(2011, 1, 21) ->added below for correct datatype
        var TITLE = '{$args['title']}'; 
        var CONTAINER = '{$args['container']}';
    
        // reset default colors 
        Highcharts.setOptions({
            colors: ['#058DC7', '#50B432', '#ED561B', '#DDDF00', '#24CBE5', '#64E572', '#FF9655', '#FFF263', '#6AF9C4'] 
        }); 
    
        // data points added below for correct datatype interpretation
        // use browser's timezone offset for date
        Highcharts.setOptions({
            global: { useUTC: false },
        });
            
            var uom_{$args['container']} = { {$uom_array_str} };
            var chart;
            $(document).ready(function() {
                chart = new Highcharts.Chart({
                    {$exporting}
                    chart: {
                        renderTo: CONTAINER,
                        zoomType: 'x',
                        spacingRight: 20,
                        height: {$height},
                        {$width}
                        animation: false
                    },
                    credits: {
                        enabled: false
                    },
                    title: {
                        text: TITLE
                    },
                    xAxis: {
                        type: 'datetime',
                        maxZoom: {$args['increment']}*1000,  // max zoom is 5 minutes 
                        title: {
                            text: null
                        }
                    },
                    yAxis: {
                        title: {
                            text: UOM  // unit of measurement from perf data 
                        },
                        startOnTick: false,
                        showFirstLabel: false
                    },
                    tooltip: {
                        shared: true,
                        useHTML: true,
                        formatter: function() {
                            html = Highcharts.dateFormat("%A %b, %e - %l:%M %p", parseInt(this.x));
                            for (var i = 0; i < this.points.length; i++) {
                                var series_name = this.points[i].series.name;
                                var name = series_name.substring(0, series_name.lastIndexOf("(")-1);
                                html += '<br><span style="color:' + this.points[i].series.color + '">\u25CF</span> <b>' + name + '</b>: ' + Math.round(this.points[i].y * 1000) / 1000 + uom_{$args['container']}[name];
                            }
                            return html;
                        }
                    },
                    legend: {
                        enabled: {$legend}
                    },
                    plotOptions: {
                        area: {
                            {$no_hover}
                            lineWidth: 1,
                            marker: {
                                enabled: false,
                                states: {
                                    hover: {
                                        enabled: true,
                                        radius: 5
                                    }
                                }
                            },
                            shadow: false,
                            states: {
                                hover: {
                                    lineWidth: 1                        
                                }
                            },
                            fillOpacity: 0.5
                        }
                    },
GRAPH;
// End heredoc syntax 
        
    // If there is actually data... ($args['nodata'] is a boolean)
    if (!empty($args['nodata']) && $args['nodata']) {
	
		// Display no data
		// TODO: Add "No Data" text to graph
		$nodata_text = "
			text = chart.renderer.text('No Data').add();
			textBBox = text.getBBox();
			tx = chart.plotLeft + (chart.plotWidth * 0.5) - (textBBox.width * 0.5);
			ty = chart.plotTop  + (chart.plotHeight * 0.5) - (textBBox.height * 0.5);
			text.attr({x: tx, y: ty});
		";
	
	} else {

        $graph .= "series: [";

        // Loop for multiple data sets in perfdata 
        $series = array(); 
        for($i=0;$i<count($args['datastrings']);$i++)
        { 
            $series[] = "
                    {
                        type: \"area\",
                        name: \"{$args['names'][$i]} (Last: ".round($perfdata_extra[$i]['avg'], 2).$units[$i].",  Avg: ".round($perfdata_extra[$i]['avg'], 2).$units[$i].",  Max: ".round($perfdata_extra[$i]['max'], 2).$units[$i].")\",                          
                        pointInterval: {$args['increment']}*1000,
                        pointStart: {$args['start']},
                        data: [
                            ".implode(', ', $args['datastrings'][$i])." 
                        ],
                        animation: false,
                        {$no_hover_fill}
                    }"; 
        }
            
        $graph .= implode(',', $series);
        $graph .= "]";
    }

    // End the highcharts graph syntax
    $graph .= " },
                function(chart) {
                    chart.title.addClass('chartbutton');
                    chart.title.on('click', function() {
                        window.location = '" . $hs_url . "';
                    })
                });             
            });";
		
	// Add nodata text if it exists
	if (isset($nodata_text)) {
		$graph .= $nodata_text;
	}
	
    return $graph;      
}

