#!/bin/sh
#
# Highcharts Exporting Server Pre-reqs
# - Installs java, java-devel, and builds the batik 1.7 libraries necessary for running the 
#   Highcharts exporting server locally for users without external access.
#
# Copyright (c) 2010-2014 Nagios Enterprises, LLC
#

# Downloads and installs the phantomjs binary (single file for 64bit or 32bit systems)
phantomjs_install() {
	if [ "$arch" == "x86_64" ]; then
		package="phantomjs-1.9.8-linux-x86_64"
	else
		package="phantomjs-1.9.8-linux-i686"
	fi

	# Download, unpack, and move
	wget http://assets.nagios.com/downloads/nagiosxi/packages/$package.tar
	tar xf $package.tar
	cp $package/bin/phantomjs $xidir/html/includes/components/highcharts/exporting-server

	# Clean up
	rm -rf phantomjs*
}

# Installs yum dependencies, downloads source, builds batik, and adds
# special policy info to th java policy
java_batik_install() {
	# Install the dependancies in yum
	echo "Installing yum dependancies..."
	yum install java-devel -y

	# Download the batik source file & uncompress
	echo "Downloading source dependancies..."
	wget http://assets.nagios.com/downloads/nagiosxi/packages/batik-src-1.7.tar.gz
	tar xvf batik-src-1.7.tar.gz
	cd batik*/batik*

	# Once inside the batik-1.7 folder let's build the libraries
	echo "Building source dependancies..."
	chmod +x -R *
	JAVA_HOME='/usr/lib/jvm/java-openjdk'
	./build.sh jars

	# Move the newly created files to the components directory
	libsdir="$xidir/html/includes/components/highcharts/exporting-server/lib"
	mkdir -p $libsdir
	chmod +x batik-1.7/batik-rasterizer.jar
	cp -f batik-1.7/batik-rasterizer.jar $xidir/html/includes/components/highcharts/exporting-server
	cp -rf batik-1.7/lib/* $libsdir
	chown nagios:nagios -R $xidir/html/includes/components/highcharts

	# Cleaning up
	cd /tmp
	rm -rf batik*

	# Add permissions to Java policy file
	echo "Adding permissions to temporary folder in java.policy... old policy file backed up..."
	cp /usr/lib/jvm/java/jre/lib/security/java.policy /usr/lib/jvm/java/jre/lib/security/java.policy.old
	echo 'grant codeBase "file:/usr/local/nagiosxi/html/includes/components/highcharts/exporting-server/-" {
	        permission java.security.AllPermission;
	};' >> /usr/lib/jvm/java/jre/lib/security/java.policy
}

###############################
# USAGE / HELP
###############################
usage () {
	echo ""
	echo "Use this script to install PhantomJS (recommended) or Java + Batik for exporting Highcharts graphs."
	echo ""
		echo " -l | --location       Location of Nagios XI basedirectory (normally to /usr/local/nagisoxi [default])"
        echo " -t | --type           Type of server to install (options: phantomjs [default] or java-batik)"
	echo ""
}

###############################
# ADDING LOGIC FOR NEW BACKUPS
###############################
while [ -n "$1" ]; do
	case "$1" in
		-h | --help)
			usage
			exit 0
			;;
		-l | --location)
			xidir=$2
			;;
		-t | --type)
			install_type=$2
			;;
	esac
	shift
done

# Start with the installation
if [ -z $xidir ]; then
	xidir="/usr/local/nagiosxi"
fi
if [ -z $install_type ]; then
	install_type="phantomjs"
fi
cd /tmp
arch=`uname -m`

# Run whichever installer we want
if [ "$install_type" == "phantomjs" ]; then
	phantomjs_install
else
	java_batik_install
fi

# Adding temp folder
echo "Creating temporary folder..."
mkdir -p $xidir/html/includes/components/highcharts/exporting-server/temp
chown nagios:nagios $xidir/html/includes/components/highcharts/exporting-server/temp
chmod 775 $xidir/html/includes/components/highcharts/exporting-server/temp

echo "Successfully added pre-reqs for local Highcharts exporting server."
exit 0